@@ -24,10 +24,11 @@ import textwrap
 import importlib
 
 from contextlib import ExitStack, contextmanager
+from collections import namedtuple
 
 import testflows.settings as settings
 
-from .exceptions import DummyTestException, ResultException, RepeatTestException, DescriptionError
+from .exceptions import DummyTestException, ResultException, TestIteration, DescriptionError
 from .flags import Flags, SKIP, TE, FAIL_NOT_COUNTED, ERROR_NOT_COUNTED, NULL_NOT_COUNTED
 from .flags import CFLAGS, PAUSE_BEFORE, PAUSE_AFTER
 from .testtype import TestType, TestSubType
@@ -186,6 +187,7 @@ class TestBase(object):
             # flag to indicate if main test called init
             self._init= False
 
+        self.io = None
         self.name = name
         if self.name is None:
             raise TypeError("name must be specified")
@@ -223,7 +225,7 @@ class TestBase(object):
         self.caller_test = None
 
     @classmethod
-    def make_name(cls, name, parent=None, args=None):
+    def make_name(cls, name, parent=None, args=None, format=True):
         """Make full name.
 
         :param name: name
@@ -233,7 +235,9 @@ class TestBase(object):
         if args is None:
             args = dict()
         try:
-            name = str(name).format(**{"$cls": cls}, **args) if name is not None else cls.name
+            name = str(name) if name is not None else cls.name
+            if name and format:
+                name = name.format(**{"$cls": cls}, **args)
         except Exception as exc:
             raise NameError(f"can't format '{name}' using {args} {str(exc)}") from None
         if name is None:
@@ -244,13 +248,16 @@ class TestBase(object):
         return join(get(parent, name_sep), name)
 
     @classmethod
-    def make_description(cls, description, args):
+    def make_description(cls, description, args, format=True):
         if args is None:
             args = dict()
         try:
-            return str(description).format(**{"$cls": cls}, **args) if description is not None else None
+            description = str(description) if description is not None else None
+            if description and format:
+                description = description.format(**{"$cls": cls}, **args)
         except Exception as exc:
             raise DescriptionError(f"can't format '{description}' using {args} {str(exc)}") from None
+        return description
 
     @classmethod
     def make_tags(cls, tags):
@@ -278,6 +285,8 @@ class TestBase(object):
             return self
 
     def _exit(self, exc_type, exc_value, exc_traceback):
+        if not self.io:
+            return False
         if top() is self and not self._init:
             return False
 
@@ -390,9 +399,6 @@ class TestDefinition(object):
         test = kwargs.pop("test", None)
         no_arguments = None
 
-        if run and test:
-            raise TypeError("can't define both test and run")
-
         if kwargs.get("args", None):
             no_arguments = False
 
@@ -444,6 +450,7 @@ class TestDefinition(object):
         self.tags = None
         self.repeat = None
         self.repeatable_func = None
+        self._with_block_frame = None
         return self
 
     def _argparser(self, argparser=None):
@@ -661,14 +668,17 @@ class TestDefinition(object):
             kwargs = self.kwargs
             kwargs["args"] = dict(kwargs.get("args") or {})
 
+            self._set_current_top_previous()
+
             argparser = kwargs.pop("argparser", None)
             parent = kwargs.pop("parent", None) or current()
             keep_type = kwargs.pop("keep_type", None)
+            format_name = kwargs.pop("format_name", True)
+            format_description = kwargs.pop("format_description", True)
 
             if not top():
                 cli_args = self._parse_cli_args(self._argparser(argparser))
                 kwargs["args"].update({k: v for k,v in cli_args.items() if not k.startswith("_")})
-            self._set_current_top_previous()
 
             test = kwargs.pop("test", None)
             kwargs_test = test
@@ -678,7 +688,7 @@ class TestDefinition(object):
             if not issubclass(test, TestBase):
                 raise TypeError(f"{test} must be subclass of TestBase")
 
-            name = test.make_name(self.name, parent.name if parent else None, kwargs["args"])
+            name = test.make_name(self.name, parent.name if parent else None, kwargs["args"], format=format_name)
 
             if parent:
                 kwargs["parent"] = parent.name
@@ -708,7 +718,7 @@ class TestDefinition(object):
                     parent.child_count += 1
 
             self.tags = test.make_tags(kwargs.pop("tags", None))
-            self.description = test.make_description(kwargs.pop("description", None), kwargs["args"])
+            self.description = test.make_description(kwargs.pop("description", None), kwargs["args"], format=format_description)
             self.parent = parent
 
             # anchor all patterns
@@ -765,9 +775,17 @@ class TestDefinition(object):
                     sys.settrace(dummy)
                     sys._getframe(1).f_trace = functools.partial(self.__repeat__, repeat, None, None, None)
                     return
-            return self.test._enter()
 
         except (KeyboardInterrupt, Exception):
+            raise
+
+        try:
+            return self.test._enter()
+        except (KeyboardInterrupt, Exception) as exc:
+            if not self.test.io:
+                raise
+            frame = inspect.currentframe().f_back
+            self._with_block_frame = (frame, frame.f_lasti, frame.f_lineno)
             self.trace = sys.gettrace()
             sys.settrace(dummy)
             sys._getframe(1).f_trace = functools.partial(self.__nop__, *sys.exc_info())
@@ -896,7 +914,7 @@ class TestDefinition(object):
 
     def __repeat__(self, repeat=None, *args):
         sys.settrace(self.trace)
-        raise RepeatTestException(repeat)
+        raise TestIteration(repeat)
 
     def __nop__(self, exc_type, exc_value, exc_tb, *args):
         sys.settrace(self.trace)
@@ -905,25 +923,66 @@ class TestDefinition(object):
     def __exit__(self, exception_type, exception_value, exception_traceback):
         frame = inspect.currentframe().f_back
 
-        if self.test is None:
-            return
+        def make_complete_traceback(exception_traceback, frame):
+            tb = namedtuple('tb', ('tb_frame', 'tb_lasti', 'tb_lineno', 'tb_next'))
 
-        if isinstance(exception_value, RepeatTestException):
+            def walk_frame(frame, tb_next=None):
+                if frame is None:
+                    return tb_next
+                if not settings.debug and frame.f_code.co_filename.endswith("testflows/_core/test.py"):
+                    tb_next = tb_next
+                else:
+                    tb_next = tb(frame, frame.f_lasti, frame.f_lineno, tb_next)
+                return walk_frame(frame.f_back, tb_next)
+
+            def walk_tb(tb_frame):
+                tb_next = None
+                if tb_frame.tb_next:
+                    tb_next = walk_tb(tb_frame.tb_next)
+
+                if tb_frame and not settings.debug and tb_frame.tb_frame.f_code.co_filename.endswith("testflows/_core/test.py"):
+                    return tb_next
+                else:
+                    return tb(tb_frame.tb_frame, tb_frame.tb_lasti, tb_frame.tb_lineno, tb_next)
+
+            tb_next = walk_tb(exception_traceback)
+            if self._with_block_frame is not None:
+                # if self._with_block_frame is set it means an exception was raised
+                # in __enter__() during test._enter() call. Now we need to fix
+                # traceback so that includes the line where "with" block is defined
+                # as it is lost
+                _frame, _lasti, _lineno = self._with_block_frame
+                if tb_next:
+                    tb_next = tb_next.tb_next
+                if tb_next:
+                    tb_next = tb_next.tb_next
+                tb_next = tb(_frame, _lasti, _lineno, tb_next)
+            tb_start = walk_frame(frame.f_back, tb_next)
+            return tb_start
+
+        if exception_value:
+            exception_traceback = make_complete_traceback(exception_traceback, frame)
+
+        if isinstance(exception_value, TestIteration):
             try:
                 repeat = exception_value.repeat
+
                 self.test._enter()
                 if self.repeatable_func is None:
                     raise Error("not repeatable")
+
                 __kwargs = dict(self.kwargs)
                 __kwargs.pop("name", None)
                 __kwargs.pop("parent", None)
                 __kwargs["type"] = TestType.Iteration
                 __args = __kwargs.pop("args", {})
+
                 if repeat.until == "fail":
                     __kwargs["flags"] = Flags(__kwargs.get("flags")) & ~TE
                 else:
                     # pass or complete
                     __kwargs["flags"] = Flags(__kwargs.get("flags")) | TE
+
                 for i in range(repeat.number):
                     with Iteration(name=f"{i}", tags=self.tags, **__kwargs, parent_type=self.test.type) as iteration:
                         if isinstance(self.repeatable_func, TestOutline):
@@ -950,7 +1009,7 @@ class TestDefinition(object):
 
         # if test did not handle the exception in _exit then re-raise it
         if exception_value and not test__exit__:
-            raise exception_value.with_traceback(exception_traceback)
+            raise exception_value
 
         if not self.parent:
             sys.exit(0 if self.test.result else 1)
@@ -1145,22 +1204,25 @@ class TestDecorator(object):
         def run(test):
             return process_func_result(self.func(test, **args))
 
+        test_running_outline = getattr(test, "_run_outline", False)
+
         if (test is None
                 or (test and test.type > self.type.type and self.type.type != TestType.Outline)
-                or (test and getattr(test, "_run_outline", False))):
+                or (test and test_running_outline)):
             kwargs = dict(self.func.kwargs)
 
             if isinstance(self, TestOutline):
-                no_arguments = (not args or getattr(test, "_run_outline_with_no_arguments", False))
+                no_arguments = not args or getattr(test, "_run_outline_with_no_arguments", False)
+                examples = test_running_outline and getattr(test, "examples") or self.examples
 
-                if no_arguments and getattr(test, "examples", self.examples):
+                if no_arguments and examples:
                     kwargs["args"] = {}
                     kwargs.pop("test", None)
 
                     _test_type = self.type(**kwargs, test=self)
 
                     def execute_examples():
-                        for example in getattr(test, "examples", self.examples):
+                        for example in examples:
                             _kwargs = dict(self.func.kwargs)
                             _kwargs["name"] = str(example)
                             _kwargs["args"] = dict(kwargs.get("args", {}))
@@ -1186,7 +1248,7 @@ class TestDecorator(object):
 
                     _test_type.repeatable_func = execute_examples
 
-                    if test and getattr(test, "_run_outline", False):
+                    if test and test_running_outline:
                         _test = test
                         execute_examples()
                     else:
@@ -1195,7 +1257,7 @@ class TestDecorator(object):
 
                     return _test
                 else:
-                    if test and getattr(test, "_run_outline", False):
+                    if test and test_running_outline:
                         return run(test)
                     else:
                         kwargs.pop("test", None)
